@@ -27,9 +27,11 @@
 
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusConnectionInterface>
-#include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusReply>
 
+#include "KDBusServiceIface.h"
+#include "FreeDesktopApplpicationIface.h"
+
 #include "config-kdbusaddons.h"
 
 #if HAVE_X11
@@ -80,7 +82,7 @@ KDBusService::KDBusService(StartupOptions options, QObject *parent)
     QDBusConnectionInterface *bus = 0;
 
     if (!QDBusConnection::sessionBus().isConnected() || !(bus = QDBusConnection::sessionBus().interface())) {
-        d->errorMessage = QString::fromLatin1("Session bus not found\n"
+        d->errorMessage = QLatin1String("Session bus not found\n"
                                         "To circumvent this problem try the following command (with Linux and bash)\n"
                                         "export $(dbus-launch)");
     }
@@ -89,11 +91,7 @@ KDBusService::KDBusService(StartupOptions options, QObject *parent)
         d->serviceName = d->generateServiceName();
         QString objectPath = QLatin1Char('/') + d->serviceName;
         objectPath.replace(QLatin1Char('.'), QLatin1Char('/'));
-        if (objectPath.contains(QLatin1Char('-'))) {
-            QString previous = objectPath;
-            objectPath.remove(QLatin1Char('-'));
-            qWarning() << "Invalid service name" << previous << "using:" << objectPath;
-        }
+        objectPath.replace(QLatin1Char('-'), QLatin1Char('_')); // see spec change at https://bugs.freedesktop.org/show_bug.cgi?id=95129
 
         if (options & Multiple) {
             const QString pid = QString::number(QCoreApplication::applicationPid());
@@ -112,19 +110,21 @@ KDBusService::KDBusService(StartupOptions options, QObject *parent)
         if (!d->registered) {
             if (options & Unique) {
                 // Already running so it's ok!
-                QDBusInterface iface(d->serviceName, objectPath);
-                iface.setTimeout(5 * 60 * 1000); // Application can take time to answer
                 QVariantMap platform_data;
                 platform_data.insert(QStringLiteral("desktop-startup-id"), QString::fromUtf8(qgetenv("DESKTOP_STARTUP_ID")));
                 if (QCoreApplication::arguments().count() > 1) {
-                    QDBusReply<int> reply = iface.call(QStringLiteral("CommandLine"), QCoreApplication::arguments(), QDir::currentPath(), platform_data);
+                    OrgKdeKDBusServiceInterface iface(d->serviceName, objectPath, QDBusConnection::sessionBus());
+                    iface.setTimeout(5 * 60 * 1000); // Application can take time to answer
+                    QDBusReply<int> reply = iface.CommandLine(QCoreApplication::arguments(), QDir::currentPath(), platform_data);
                     if (reply.isValid()) {
                         exit(reply.value());
                     } else {
                         d->errorMessage = reply.error().message();
                     }
                 } else {
-                    QDBusReply<void> reply = iface.call(QStringLiteral("Activate"), platform_data);
+                    OrgFreedesktopApplicationInterface iface(d->serviceName, objectPath, QDBusConnection::sessionBus());
+                    iface.setTimeout(5 * 60 * 1000); // Application can take time to answer
+                    QDBusReply<void> reply = iface.Activate(platform_data);
                     if (reply.isValid()) {
                         exit(0);
                     } else {
